module Bindep
  class Context
    attr_accessor :no_install, :no_confirm_before_install, :silent_exceptions

    # Create a new Bindep context with its own options and item registry.
    def initialize
      @registry = {}

      @no_install = false
      @no_confirm_before_install = false
      @silent_exceptions = false

      unless Helpers.which_test
        error "Command detection does not work on your system. Bindep cannot run on your system!"
      end
    end

    # Define a new item.
    def define(id, &block)
      item = Item.new(id)
      yield item

      @registry[item.id] = item
    end

    # Check if a given item is installed, can also define item beforehand if block is given.
    def check(id, force_install = false, &block)
      item = block_given? ? define(id, &block) : get_item(id)

      [ item.depends ].flatten.compact.each { |dep| check dep }
      
      install item if force_install || item.local_command.nil?
    end

    # Runs the specified command with the given arguments.
    def run(id, args = [], stdin = nil, raise_on_failure = true)
      check id

      cmd_string = "#{get_item(id).local_command} #{[ args ].flatten.join(" ")}".strip
      Helpers.cmd cmd_string, stdin, raise_on_failure
    end

    # Gets an item from the registry.
    def get_item(id)
      @registry[id.to_sym] || raise(Error, "Cannot find item '#{id.to_sym}' in context!")
    end

    private
    def pre_install(item)
      if no_install
        error "Cannot find command for '#{item.id}' and installation is deactivated. "\
        "Please install manually."
      end

      if item.install_command.nil?
        error "Cannot find installation instructions for '#{item.id}'. "\
        "Please install manually."
      end

      unless no_confirm_before_install
        puts "[Bindep] Command '#{item.id}' is not installed. "\
        "Do you want to run the following command? (y/n)"
      end

      puts "[Bindep] $ #{item.install_command}\n"

      unless no_confirm_before_install
        error "Aborting." unless STDIN.gets.downcase.strip == 'y'
      end
    end

    def install(item)
      pre_install item

      Helpers.cmd_interactive item.install_command

      unless item.local_command true
        error "Installation for '#{item.id}' failed. Please install manually."
      end
    end

    def error(message = nil)
      exception = Error.new message

      raise exception if silent_exceptions

      puts exception.to_s
      exit 1
    end
  end
end
